import React, { Component, Fragment } from 'react';
import TabContent from 'reactstrap/lib/TabContent';
import TabPane from 'reactstrap/lib/TabPane';
import Nav from 'reactstrap/lib/Nav';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { F, path } from 'ramda';
import { TabLabel } from './tab-label/tab-label';
import { NavWrapper } from './nav-wrapper/nav-wrapper';

export class Tabs extends Component {
  static propTypes = {
    activeTab: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.number,
    ]),
    children: PropTypes.node.isRequired,
    upper: PropTypes.bool,
    border: PropTypes.bool,
    flexItems: PropTypes.bool,
    onTabSelect: PropTypes.func,
    className: PropTypes.string,
    tabClass: PropTypes.string,
    contentClass: PropTypes.string,
  };

  static defaultProps = {
    activeTab: 0,
    upper: false,
    border: false,
    flexItems: false,
    className: null,
    tabClass: null,
    contentClass: null,
    onTabSelect: F,
  };

  constructor(props) {
    super(props);

    let { activeTab } = this.props;
    const firstTabId = path(['0', 'props', 'tabId'], this.props.children);

    if (firstTabId && !activeTab) {
      activeTab = firstTabId;
    }

    this.state = { activeTab };
  }

  toggle = (tabId) => {
    this.setState({ activeTab: tabId });
    this.props.onTabSelect(tabId);
  }

  renderTitles() {
    const {
      upper,
      border,
      flexItems,
      className,
      tabClass,
    } = this.props;
    const classNames = classnames(
      className,
      'navbar-nav flex-row',
      {
        'tabs-upper': upper,
        'tabs-border': border,
        'tabs-flex-items': flexItems,
      }
    );

    return (
      <NavWrapper>
        <Nav tabs className={classNames}>
          {this.props.children.map((child, i) => {
            const { label, tabId } = child.props;
            const currentTabId = tabId || i;

            return label ? (
              <TabLabel
                key={label}
                label={label}
                tabId={currentTabId}
                isActive={this.state.activeTab === currentTabId}
                className={tabClass}
                onClick={this.toggle}
              />
            ) : React.cloneElement(child); // For cases when we need custom element in navbar (along-with TabLabel)
          })}
        </Nav>
      </NavWrapper>
    );
  }

  render() {
    const { children, contentClass } = this.props;

    return (
      <Fragment>
        {this.renderTitles()}
        <TabContent className={contentClass} activeTab={this.state.activeTab}>
          {children.map((child, i) => {
            const { tabId, label, className } = child.props;
            return label ?
              React.cloneElement(child, { tabId: tabId || i, key: label, className }) :
              null; // No content for custom elements
          })}
        </TabContent>
      </Fragment>
    );
  }
}

export const Tab = ({ children, tabId, className }) => (
  <TabPane tabId={tabId} className={className}>
    {children}
  </TabPane>
);

Tab.propTypes = {
  children: PropTypes.node.isRequired,
  tabId: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.number,
  ]),
  className: PropTypes.string,
};

Tab.defaultProps = {
  tabId: null,
  className: null,
};
